@@ -257,7 +257,8 @@ let rec pattern_p m =
       begin
 	match (!x) with
 	| None -> distinct_bvar_list_p ml
-	| Some n -> raise NotGround (*** Actually, this is not yet written - just fail for now ***)
+	| Some n ->
+	    raise (Failure "pattern_p case missing") (*** Actually, this is not yet written - just fail for now ***)
       end
   | MLam(a,m1) -> pattern_p m1
   | MAp(m1,m2) -> pattern_p m1 && pattern_p m2
