@@ -181,10 +181,9 @@ let (s2,_,w2,_,_,_,_) = statistic1 r 0 false in
 
 (** MAIN **)    
 
-
-let print_coq_proofterm c r =
+let print_coq_proofscript c r =
   
-  if (!verbosity > vold) then Printf.printf  "starting print_coq_proofterm.\n";
+  if (!verbosity > vold) then Printf.printf  "starting print_coq_proofscript.\n";
   if (!verbosity > vold) then print_refut r;
 
 	(*** Search ***)
@@ -238,7 +237,83 @@ let print_coq_proofterm c r =
 	(*** Output Coq ***)
 
   let beforeCoq= Sys.time() in
-  if (result_coq && size < 800 ) then ref_coq c prenorm_refutation else raise (CoqProofTooBig size);
+  if (result_coq && size < maxcoqproofsize ) then ref_coq c prenorm_refutation else raise (CoqProofTooBig size);
+  let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
+  if (result_coq ) then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
+	(*** Output Latex Search ***)
+
+  if (result_latex && width < 50 && depth < 30) then
+  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n ***)\n" 
+  size depth width (ref_to_lat initbranch refutation)
+  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d \n \n  %%endLatex \n ***)\n" size depth width;
+ 
+	(*** Output Latex Translation ***)
+
+  if (result_latex &&  width < 50 && depth < 30) then
+  Printf.fprintf c "(*** \n %%beginLatex \n size %d depth %d width %d cuts %d rewrite  %d NYI %d \n \n \\begin{tabular}{c} \n %s \\end{tabular} \n\n %%endLatex \n***)" 
+  size depth width cut rewrite notyetImpl  (ref_to_lat initbranch_prenorm prenorm_refutation)
+  else if (result_latex) then Printf.fprintf c "(*** \n %%beginLatex \n \n \n Translation successful, probably \n \n size %d depth %d width %d cuts %d rewrite %d  NYI %d \n %%endLatex \n***)" size depth width cut rewrite notyetImpl 
+  ; flush stdout
+  end
+  end		
+
+let print_coq_sproofterm c r =
+  
+  if (!verbosity > vold) then Printf.printf "starting print_coq_proofspfterm.\n";
+  if (!verbosity > vold) then print_refut r;
+
+	(*** Search ***)
+
+  (* get assumptions *)
+  let initbranch1 = (List.rev !initial_branch) in
+  let initbranch = (List.map preprocess initbranch1) in (*** Chad, Aug 2011: Preprocess to handle LEIBEQ_TO_PRIMEQ here as well as in suche.ml. Example: SYO348^5.p ***)
+  let b =List.map get_literal initbranch in
+  (* call search *)
+  let (refutation,con,searchTime,isTimeout) = search_refutation b r in
+  (* check wether the dependency of the returned refutation is a subset of the assumptions - fail indicates some bug*)
+  if assert_condition && not (Dependency.is_empty (Dependency.diffunion [] [con] [b]) ) then 
+    (if (!verbosity > 0) then Printf.printf  "Error with refutation: Still open conditions  \n") ;	 
+  let (size,_,depth,width,_,cut,rewrite,notyetImpl,timeouts) = statistic refutation in
+  (* if timeout was reached the refutation is unfinished - at least output some information about the completed part*)
+	if isTimeout  || timeouts > 0 then  
+  	  (if (!verbosity > 0) then Printf.printf  "Timeout!  time %d ms size %d depth %d width %d cuts %d rewrite %d NYI %d timeouts %d \n" searchTime size depth width cut rewrite notyetImpl timeouts)
+  	else 	
+  begin
+    if result_print_search then Printf.printf  "%s\n" (ref_str refutation);
+    if (!verbosity > 3) then Printf.printf "Search time %d ms size %d depth %d width %d \n" searchTime size depth width ;
+
+	(*** Translation  ***)
+
+(* get prenormalized assumptions *)
+  let initbranch_prenorm = List.map 
+	(fun pn ->let p= onlynegnorm pn in  if debug_translation then Printf.printf  "%s = %s \n" (trm_str pn) (trm_str p) ;p ) 
+	(List.rev !initial_branch_prenorm) in
+  let beforeTrans= Sys.time() in
+(* Branch is an association list from normalized terms to their prenormalized counterparts *)
+  let branch = (List.combine initbranch  initbranch_prenorm) in
+  if debug_translation then (Printf.printf "About to translate. Branch assoc with prenorm:\n"; List.iter (fun (m,p) -> Printf.printf "trm: %s\npre: %s\n" (trm_str m) (trm_str p)) branch);
+(* The flag pftrm_eager_rewrite decides wether rewrites are eagerly or lazily handled *) 
+  let prenorm_refutation =
+    begin
+      try
+	if pftrm_eager_rewrite then
+	  eager_translate branch refutation
+	else
+	  lazy_translate branch refutation
+      with TranslateFailure -> safe_translate branch refutation (*** Chad Aug 2011 : Fall back on safe_translate ***)
+    end
+  in
+  if debug_translation then (Printf.printf "Translated prenorm_refutation\n%s\n" (ref_str prenorm_refutation));
+  let transTime= int_of_float ((Sys.time() -. beforeTrans) *. 1000.0) in
+  begin
+  let (size,_,depth,width,_,cut,rewrite,notyetImpl,_) = statistic prenorm_refutation  in
+  if result_print_translation then Printf.printf  "%s\n" (ref_str prenorm_refutation);
+  if (!verbosity > 3) then Printf.printf  "Translation NYI %d time %d ms size %d depth %d width %d cuts %d rewrite %d  \n" notyetImpl transTime size depth width cut rewrite  ;
+
+	(*** Output Coq ***)
+
+  let beforeCoq= Sys.time() in
+  if (result_coq && size < maxcoqproofsize ) then ref_coq_spfterm c prenorm_refutation else raise (CoqProofTooBig size);
   let coqTime= int_of_float ((Sys.time() -. beforeCoq) *. 1000.0) in
   if (result_coq ) then if (!verbosity > 3) then Printf.printf  "Coq output done: time %d  \n" coqTime ;
 	(*** Output Latex Search ***)
@@ -259,7 +334,7 @@ let print_coq_proofterm c r =
   end		
   
 (*** Chad, August 2011: Added this way of printing proof information at the request of Jasmin Blanchette. ***)
-let print_hocore_proofterm c r =
+let print_hocore c r =
   let don : (string,unit) Hashtbl.t = Hashtbl.create 1000 in
   let report_name name =
     if (not (Hashtbl.mem don name)) then
@@ -268,15 +343,30 @@ let print_hocore_proofterm c r =
 	Hashtbl.add don name ()
       end
   in
-  let rec hocore_proofterm_1 c r a =
+  let rec hocore_1 c r a =
+    if (!verbosity > 50) then
+      begin
+	Printf.printf "hocore_1 r\n";
+	print_refut r;
+	Printf.printf "hocore_1 a BEGIN\n";
+	List.iter (fun (m,n) -> Printf.printf "%s : %s\n" n (trm_str m)) a;
+	Printf.printf "hocore_1 a END\n";
+      end;
     match r with
     | SearchR(cl,cr) ->
 	List.iter
 	  (fun c0 ->
 	    try
+	      if (!verbosity > 50) then
+		begin
+		  Printf.printf "Calling cr on clause ";
+		  List.iter (fun i -> Printf.printf "%d\n" i) c0;
+		  Printf.printf "\n";
+		end;
 	      match (cr c0) with
 	      |	ChoiceRule(Name(eps,_),p) ->
 		  begin
+		    if (!verbosity > 50) then Printf.printf "cr found it's a choice rule\n";
 		    try
 		      let (_,choiceax) = Hashtbl.find choiceopnames eps in
 		      let name = List.assoc choiceax a in
@@ -284,10 +374,14 @@ let print_hocore_proofterm c r =
 		    with
 		      Not_found -> ()
 		  end
-	      |	_ -> ()
+	      |	ri ->
+		  begin
+		    if (!verbosity > 50) then Printf.printf "cr found it has rule info %s\n" (ruleinfo_str ri);
+		  end
 	    with (** Assumptions have no associated rule info **)
 	      Not_found ->
 		begin
+		  if (!verbosity > 50) then Printf.printf "cr found no rule info\n";
 		  match c0 with
 		  | [l] ->
 		      begin
@@ -322,13 +416,13 @@ let print_hocore_proofterm c r =
 	  | [] -> raise (GenericError "Assumption Conflict Not Found")
 	in
 	find_conflict a
-    | FalseR -> (*** Do this ***)
+    | FalseR ->
 	begin
 	  try
 	    report_name (List.assoc False a)
 	  with Not_found -> raise (GenericError "Assumption 'False' Not Found")
 	end
-    | NegReflR -> (*** Do this ***)
+    | NegReflR ->
 	let rec find_conflict a =
 	  match a with
 	  | (((Ap(Ap(Imp,Ap(Ap(Eq(_),m),n)),False)),name)::_) when (m = n) -> report_name name
@@ -341,60 +435,62 @@ let print_hocore_proofterm c r =
 	begin
 	  try
 	    let name = List.assoc (neg (imp m n)) a in
-	    hocore_proofterm_1 c r ((m,name)::(normneg n,name)::a)
+	    hocore_1 c r ((m,name)::(normneg n,name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (neg (imp m n)))))
 	end
     | ImpR(m,n,r1,r2) ->
 	begin
 	  try
 	    let name = List.assoc (imp m n) a in
-	    hocore_proofterm_1 c r1 ((normneg m,name)::a);
-	    hocore_proofterm_1 c r2 ((n,name)::a)
+	    hocore_1 c r1 ((normneg m,name)::a);
+	    hocore_1 c r2 ((n,name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (imp m n))))
 	end
     | NegAllR(b,m,x,r) -> 
 	begin
 	  try
 	    let name = List.assoc (neg (Ap(Forall(b),m))) a in
-	    hocore_proofterm_1 c r (((norm name_def (Ap(m,Name(x,b)))),name)::a)
+	    hocore_1 c r (((norm name_def (neg (Ap(m,Name(x,b))))),name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (neg (Ap(Forall(b),m))))))
 	end
     | EqFR(a1,a2,m,n,r) ->  (*** This doesn't appear to be used at the moment. - Chad, Aug 2011 ***)
 	begin
 	  try
 	    let name = List.assoc (Ap(Ap(Eq(Ar(a1,a2)),m),n)) a in
-	    hocore_proofterm_1 c r (((norm name_def (forall a1 (eq a2 (Ap(m,DB(0,a1))) (Ap(n,DB(0,a1)))))),name)::a)
+	    hocore_1 c r (((norm name_def (forall a1 (eq a2 (Ap(m,DB(0,a1))) (Ap(n,DB(0,a1)))))),name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (Ap(Ap(Eq(Ar(a1,a2)),m),n)))))
 	end
     | NegEqFR(a1,a2,m,n,r) -> 
 	begin
 	  try
 	    let name = List.assoc (neg (Ap(Ap(Eq(Ar(a1,a2)),m),n))) a in
-	    hocore_proofterm_1 c r (((norm name_def (normneg (forall a1 (eq a2 (Ap(m,DB(0,a1))) (Ap(n,DB(0,a1))))))),name)::a)
+	    hocore_1 c r (((norm name_def (normneg (forall a1 (eq a2 (Ap(m,DB(0,a1))) (Ap(n,DB(0,a1))))))),name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (neg (Ap(Ap(Eq(Ar(a1,a2)),m),n))))))
 	end
     | EqOR(m,n,r1,r2) -> 
 	begin
 	  try
 	    let name = List.assoc (eq Prop m n) a in
-	    hocore_proofterm_1 c r1 ((m,name)::(normneg n,name)::a);
-	    hocore_proofterm_1 c r2 ((normneg m,name)::(n,name)::a)
+	    hocore_1 c r1 ((m,name)::(normneg n,name)::a);
+	    hocore_1 c r2 ((normneg m,name)::(n,name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (eq Prop m n))))
 	end
     | NegEqOR(m,n,r1,r2) -> 
 	begin
 	  try
 	    let name = List.assoc (neg (eq Prop m n)) a in
-	    hocore_proofterm_1 c r1 ((m,name)::(normneg n,name)::a);
-	    hocore_proofterm_1 c r2 ((normneg m,name)::(n,name)::a)
+	    hocore_1 c r1 ((m,name)::(normneg n,name)::a);
+	    hocore_1 c r2 ((normneg m,name)::(n,name)::a)
 	  with Not_found -> raise (GenericError ("Unknown Assumption " ^ (trm_str (neg (eq Prop m n)))))
 	end
   in
   let a = Hashtbl.fold (fun k v l -> (v,k)::l) name_hyp []
   in
-  hocore_proofterm_1 c r
+  hocore_1 c r
     begin
       match !conjecture with
-      | Some (_,_,ntmn) -> ((ntmn,!conjecturename)::a)
+      | Some (_,_,ntmn) ->
+	  if (!verbosity > 20) then Printf.printf "neg conj name %s %s\n" !conjecturename (trm_str ntmn);
+	  ((ntmn,!conjecturename)::a)
       | _ -> a
     end
