@@ -6,20 +6,36 @@ open String
 open Flags
 open Syntax
 open Priorityqueue
+open Minisatinterface
 
 exception Redeclaration of string
 exception FileNotFound of string
 exception GenericError of string
 exception CoqProofTooBig of int
 
+let slaveargs = ref [Sys.argv.(0)];;
 let mode : string list ref = ref []
 let timeout : float option ref = ref None
 let verbosity : int ref = ref 1 (*** Default to 1; 0 should mean print nothing (-verb 0 command line option) ***)
 let coq = ref false
-let coqchannel : out_channel option ref = ref None
+let coq2 = ref false
+let coqlocalfile = ref false
+let coqglobalfile = ref false
+let coqinchannel : in_channel ref = ref stdin
+let coqoutchannel : out_channel ref = ref stdout
+let coqinticks : ((out_channel -> unit) option * int) list ref = ref []
+let coqoutfn1 = ref (fun c -> ())
+let coqctx : (string option * pretrm option * pretrm option) list ref = ref []
+let coqglobalsectionstack : (string * (out_channel -> unit) * (string option * pretrm option * pretrm option) list) list ref = ref []
+
+let rec updatecoqglobalsectionstack cx cgss co =
+  match cgss with
+  | ((secname,cfn,lcx)::cgss') -> ((secname,co cx cfn,lcx)::(updatecoqglobalsectionstack (List.append cx lcx) cgss' co))
+  | [] -> []
+
 let conjecturename : string ref = ref "claim"
 let conjecture : (pretrm * trm * trm) option ref = ref None
-type proofkind = CoqScript | HOCore
+type proofkind = CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
 let mkproofterm = ref None
 let mkprooftermp () = match !mkproofterm with Some _ -> true | None -> false
 let slave1 = ref false
@@ -40,6 +56,37 @@ let coq_used_names : (string,unit) Hashtbl.t = Hashtbl.create 100
 let coq_names : (string,string) Hashtbl.t = Hashtbl.create 100
 let coq_hyp_names : (string,string) Hashtbl.t = Hashtbl.create 100
 
+let coqknown (x,y) =
+  if (!coq2) then y else x
+
+let requireSet0a () =
+  let a = Base "set" in
+  let b = PName "set" in
+  Hashtbl.add coq_used_names "In" ();
+  Hashtbl.add coq_used_names "Subq" ();
+  Hashtbl.add coq_names "In" "In";
+  Hashtbl.add coq_names "Subq" "Subq";
+  coqsig_const := ("In",Ar(a,Ar(a,Prop)))::!coqsig_const;
+  coqsig_const := ("Subq",Ar(a,Ar(a,Prop)))::!coqsig_const;
+  coqsig_def := ("Subq",PLam([("x",b);("y",b)],PAll(["z",b],PAp(PAp(PImplies,PAp(PAp(PName "In",PName "z"),PName "x")),PAp(PAp(PName "In",PName "z"),PName "y")))))::!coqsig_def;
+  coqsig_def_trm := ("Subq",Lam(a,Lam(a,Ap(Forall(a),Lam(a,Ap(Ap(Imp,Ap(Ap(Name("In",Ar(a,Ar(a,Prop))),DB(0,a)),DB(2,a))),Ap(Ap(Name("In",Ar(a,Ar(a,Prop))),DB(0,a)),DB(1,a))))))))::!coqsig_def_trm;
+  ()
+
+let required : string ref = ref ""
+
+let require x =
+  if (!verbosity > 0) then Printf.printf "Requiring %s\n" x;
+  required := x;
+  let f = !coqoutfn1 in
+  begin
+    coqoutfn1 := (fun c -> f c; Printf.fprintf c "Require Export %s.\n" x);
+    match x with
+    | "set0a" -> requireSet0a()
+    | "set0" -> raise (GenericError "set0 is not yet supported.")
+    | "set1" -> raise (GenericError "set1 is not yet supported.")
+    | _ -> ()
+  end
+
 (*** March 31, 2011 - Chad - THF policy regarding definitions. (See comments before declare_definition_real below. ***)
 exception TreatAsAssumption
 
@@ -280,7 +327,7 @@ let insertWithPriority n o =
     end
   in
   begin
-    if (!verbosity > 9) then Printf.printf "Inserting option with priority %d: %s\n" n (searchoption_str o);
+    if (!verbosity > 8) then Printf.printf "Inserting option with priority %d: %s\n" n (searchoption_str o);
     ins n o;
     if (!verbosity > 49) then dp searchoption_str
   end
@@ -505,6 +552,7 @@ let coq_init () =
     Hashtbl.add coq_used_names "SType" ();
     Hashtbl.add coq_used_names "Sar" ();
     Hashtbl.add coq_used_names "o" ();
+    Hashtbl.add coq_used_names "prop" ();
     Hashtbl.add coq_used_names "Sepsilon" ();
     Hashtbl.add coq_used_names "forall" ();
     Hashtbl.add coq_used_names "exists" ();
@@ -551,7 +599,7 @@ let print_coqsig c =
     match l with
       (x::r) ->
 	print_coqsig_base r;
-	Printf.fprintf c "Variable %s:SType.\n" x
+	if (not (!coq2)) then Printf.fprintf c "Variable %s:SType.\n" x
     | [] -> ()
   in
   let rec print_coqsig_const l =
@@ -566,7 +614,7 @@ let print_coqsig c =
 	      begin
 		try
 		  Printf.fprintf c "Variable %s : " x;
-		  print_stp_coq c a coq_names false;
+		  if (!coq2) then print_stp_coq2 c a false else print_stp_coq c a coq_names false;
 		  Printf.fprintf c ".\n"
 		with
 		| Not_found ->
@@ -587,9 +635,9 @@ let print_coqsig c =
 	    let m = List.assoc x (!coqsig_def) in
 	    begin
 	      Printf.fprintf c "Definition %s : " x;
-	      print_stp_coq c a coq_names false;
+	      if (!coq2) then print_stp_coq2 c a false else print_stp_coq c a coq_names false;
 	      Printf.fprintf c "\n := ";
-	      print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
+	      if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
 	      Printf.fprintf c ".\n"
 	    end
 	  with
@@ -604,7 +652,7 @@ let print_coqsig c =
 	  try
 	    print_coqsig_hyp r;
 	    Printf.fprintf c "Hypothesis %s : " x;
-	    print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
+	    if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
 	    Printf.fprintf c ".\n"
 	  with
 	  | Not_found ->
@@ -615,10 +663,22 @@ let print_coqsig c =
 	end
     | [] -> ()
   in
+  if (not (!coqlocalfile)) then
   begin
-    Printf.fprintf c "Add LoadPath \"%s/coq\".\n" Config.satallaxdir;
-    Printf.fprintf c "Require Import stttab.\n";
-    Printf.fprintf c "Section SatallaxProblem.\n";
+    begin
+      match (!mkproofterm) with
+      |	Some CoqSPfTerm ->
+	  begin
+	    Printf.fprintf c "Require Export stt3.\n";
+	  end
+      |	Some CoqScript ->
+	  begin
+	    Printf.fprintf c "Add LoadPath \"%s/coq\".\n" Config.satallaxdir;
+	    Printf.fprintf c "Require Import stttab.\n";
+	    Printf.fprintf c "Section SatallaxProblem.\n"
+	  end
+      |	_ -> ()
+    end;
     print_coqsig_base !coqsig_base;
     print_coqsig_const !coqsig_const;
     print_coqsig_def !coqsig_const;
@@ -627,7 +687,7 @@ let print_coqsig c =
     | Some (m,_,_) ->
 	begin
 	  Printf.fprintf c "Theorem %s : " (Hashtbl.find coq_hyp_names (!conjecturename));
-	  print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
+	  if (!coq2) then print_pretrm_coq2 c m (-1) (-1) else print_pretrm_coq c m coq_names coq_used_names (-1) (-1);
 	  Printf.fprintf c ".\n"
 	end
     | None -> Printf.fprintf c "Theorem claim : False.\n"
@@ -750,7 +810,7 @@ let declare_definition_real x m =
   with
   | Not_found ->
       begin (*** Giving a definition without giving it's type isn't allowed in THF anymore.  I'm still allowing it.  ***)
-	if (!verbosity > 0) then Printf.printf "WARNING: %s defined without giving type first.\n" x;
+	if ((!verbosity > 0) && (not (!coqglobalfile)) && (not (!coqlocalfile))) then Printf.printf "WARNING: %s defined without giving type first.\n" x;
 	let (tm,tp) = st_to_trm m in
 	if (!coq) then
 	  begin
@@ -780,6 +840,7 @@ let rec declare_thf_logic_formula (name:string) (role:string) (m:pretrm) =
 	  end;
 	let tmn = norm name_def (logicnorm tm) in
 	Hashtbl.add name_hyp name tmn;
+	if (!verbosity > 20) then Printf.printf "name_hyp %s %s\n" name (trm_str tmn);
 	initial_branch_prenorm := (tm::(!initial_branch_prenorm));
 	initial_branch := (tmn::(!initial_branch))
       end
@@ -903,3 +964,17 @@ let reset_search () =
     search_init()
   end
 
+let print_model b =
+  Hashtbl.iter
+    (fun m i ->
+      let v = minisat_modelValue i in
+      if (b) then
+	begin
+	  if (v = 0) then (*** 0 means true ***)
+	    Printf.printf "%d %s\n" i (trm_str m);
+	end
+      else
+	if (v < 2) then
+	  Printf.printf "%d%s %s\n" i (if (v = 0) then "+" else "-") (trm_str m)
+      )
+    atom_hash
