@@ -11,14 +11,25 @@ exception FileNotFound of string
 exception GenericError of string
 exception CoqProofTooBig of int
 
+val slaveargs : string list ref
 val mode : string list ref
 val timeout : float option ref
 val verbosity : int ref
 val coq : bool ref
-val coqchannel : out_channel option ref
+val coq2 : bool ref
+val coqlocalfile : bool ref
+val coqglobalfile : bool ref
+val coqinchannel : in_channel ref
+val coqoutchannel : out_channel ref
+val coqinticks : ((out_channel -> unit) option * int) list ref
+val coqoutfn1 : (out_channel -> unit) ref
+val coqctx : prectx ref
+val coqglobalsectionstack : (string * (out_channel -> unit) * prectx) list ref
+val updatecoqglobalsectionstack : prectx -> (string * (out_channel -> unit) * prectx) list -> (prectx -> (out_channel -> unit) -> (out_channel -> unit)) -> (string * (out_channel -> unit) * prectx) list
+
 val conjecturename : string ref
 val conjecture : (pretrm * trm * trm) option ref
-type proofkind = CoqScript | HOCore
+type proofkind = CoqScript | CoqSPfTerm | HOCore | Model | ModelTrue
 val mkproofterm : proofkind option ref
 val mkprooftermp : unit -> bool
 val slave1 : bool ref
@@ -38,6 +49,10 @@ val name_hyp : (string,trm) Hashtbl.t
 val coq_used_names : (string,unit) Hashtbl.t
 val coq_names : (string,string) Hashtbl.t
 val coq_hyp_names : (string,string) Hashtbl.t
+val coqknown : string * string -> string
+
+val required : string ref
+val require : string -> unit
 
 val get_fresh_name : stp -> string * trm
 
@@ -216,4 +231,6 @@ val search_init : unit -> unit
 (*** Reset Search ***)
 val reset_search : unit -> unit
 
+val print_model : bool -> unit
+
 
