@@ -14,13 +14,43 @@ type pretrm =
   | PInt of int
   | PReal of float
   | PAp of pretrm * pretrm
+  | PULam of string list * pretrm
   | PLam of ((string * pretrm) list) * pretrm
   | PAll of ((string * pretrm) list) * pretrm
   | PEx of ((string * pretrm) list) * pretrm
+  | PExU of string * pretrm * pretrm
   | PChoice of (string * pretrm) * pretrm
   | PAr of pretrm * pretrm
   | POf of pretrm * pretrm
   | PDef of pretrm * pretrm
+  | PLet of string * pretrm * pretrm
+  | PTLet of string * pretrm * pretrm * pretrm
+
+type prectx = (string option * pretrm option * pretrm option) list
+
+let rec prectx_lam c m =
+  match c with
+  | (Some x,Some a,None)::c' -> prectx_lam c' (PLam([(x,a)],m))
+  | (Some x,_,Some n)::c' -> prectx_lam c' (PLet(x,n,m))
+  | (None,Some p,None)::c' -> prectx_lam c' m
+  | _::_ -> raise (GenericSyntaxError "Impossible prectx case")
+  | [] -> m
+
+let rec prectx_all c m =
+  match c with
+  | (Some x,Some a,None)::c' -> prectx_all c' (PAll([(x,a)],m))
+  | (Some x,_,Some n)::c' -> prectx_all c' (PLet(x,n,m))
+  | (None,Some p,None)::c' -> prectx_all c' (PAp(PAp(PImplies,p),m))
+  | _::_ -> raise (GenericSyntaxError "Impossible prectx case")
+  | [] -> m
+
+let rec prectx_ar c m =
+  match c with
+  | (Some x,Some a,None)::c' -> prectx_ar c' (PAr(a,m))
+  | (Some x,_,Some n)::c' -> prectx_ar c' m
+  | (None,Some p,None)::c' -> prectx_ar c' m
+  | _::_ -> raise (GenericSyntaxError "Impossible prectx case")
+  | [] -> m
 
 let rec pretrm_str m =
   match m with
@@ -48,13 +78,17 @@ let rec pretrm_str m =
   | PInt i -> string_of_int i
   | PReal z -> string_of_float z
   | PAp(n,p) -> "(" ^ (pretrm_str n) ^ " @ " ^ (pretrm_str p) ^ ")"
+  | PULam(xl,m) -> "(^ [" ^ (String.concat "," xl) ^ "] " ^ (pretrm_str m) ^ ")"
   | PLam(xl,m) -> "(^ [" ^ (String.concat "," (List.map (fun (x,a) -> x ^ ":" ^ (pretrm_str a)) xl)) ^ "] " ^ (pretrm_str m) ^ ")"
   | PAll(xl,m) -> "(! [" ^ (String.concat "," (List.map (fun (x,a) -> x ^ ":" ^ (pretrm_str a)) xl)) ^ "] " ^ (pretrm_str m) ^ ")"
   | PEx(xl,m) -> "(? [" ^ (String.concat "," (List.map (fun (x,a) -> x ^ ":" ^ (pretrm_str a)) xl)) ^ "] " ^ (pretrm_str m) ^ ")"
+  | PExU(x,a,m) -> "(?! [" ^ x ^ ":" ^ (pretrm_str a) ^ "] " ^ (pretrm_str m) ^ ")"
   | PChoice((x,xa),m) -> "(@+ [" ^ x ^ ":" ^ (pretrm_str xa) ^ "] " ^ (pretrm_str m) ^ ")"
   | PAr(n,p) -> "(" ^ (pretrm_str n) ^ " > " ^ (pretrm_str p) ^ ")"
   | POf(n,p) -> "(" ^ (pretrm_str n) ^ " : " ^ (pretrm_str p) ^ ")"
   | PDef(n,p) -> "(" ^ (pretrm_str n) ^ " := " ^ (pretrm_str p) ^ ")"
+  | PLet(x,n,p) -> "(" ^ x ^ " := " ^ (pretrm_str n) ^ " in " ^ (pretrm_str p) ^ ")"
+  | PTLet(x,a,n,p) -> "(" ^ x ^ " : " ^ (pretrm_str a) ^ " := " ^ (pretrm_str n) ^ " in " ^ (pretrm_str p) ^ ")"
 
 type input =
     DeclareTHF of string * string * pretrm
@@ -105,7 +139,7 @@ let rpar p = if p then ")" else ""
 let rec stp_str_rec a p =
   match a with
     Base(x) -> x
-  | Prop -> "o"
+  | Prop -> "prop"
   | Ar(b,c) -> (lpar p) ^ (stp_str_rec b true) ^ ">" ^ (stp_str_rec c false) ^ (rpar p)
 
 let stp_str a = stp_str_rec a false
@@ -447,6 +481,15 @@ let rec print_pretrm_coq c m h hu lp rp =
 	end
   | POf(n,p) -> print_pretrm_coq c n h hu lp rp
 (***  | PDef(n,p) -> print_pretrm_coq c n h hu lp rp ***)
+  | PULam(xl,m) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	begin
+	  Printf.fprintf c "fun";
+	  print_ulam_coq c xl m h hu
+	end;
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
   | PLam(xl,m) ->
       begin
 	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
@@ -495,6 +538,26 @@ let rec print_pretrm_coq c m h hu lp rp =
 	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
       end
   | _ -> raise (GenericSyntaxError ("Unknown pretrm case print Coq version : " ^ (pretrm_str m)))
+and print_ulam_coq c xl m h hu =
+  match xl with
+    (x::xr) ->
+      begin
+	let y = coqify_name x h hu in
+	Hashtbl.remove h x; (*** pops y from the values of x to print domain ***)
+	Hashtbl.remove hu y; (*** free y to print the domain ***)
+	Printf.fprintf c " ";
+	Printf.fprintf c "%s" y;
+	Hashtbl.add h x y; (*** put y back on ***)
+	Hashtbl.add hu y (); (*** put y back on ***)
+	print_ulam_coq c xr m h hu;
+	Hashtbl.remove h x; (*** pops y from the values of x ***)
+	Hashtbl.remove hu y; (*** free y to be used later ***)
+      end
+  | [] ->
+      begin
+	Printf.fprintf c " => ";
+	print_pretrm_coq c m h hu (-1) (-1)
+      end
 and print_lam_coq c xl m h hu =
   match xl with
     ((x,a)::xr) ->
@@ -552,6 +615,373 @@ and print_ex_coq c xl m h hu =
 	print_pretrm_coq c m h hu (-1) (-1)
       end
 
+let rec print_stp_coq2 c m p =
+  match m with
+  | Base _ ->
+      Printf.fprintf c "set"
+  | Prop ->
+      Printf.fprintf c "prop"
+  | Ar(a,b) ->
+      begin
+	if p then Printf.fprintf c "(";
+	print_stp_coq2 c a true;
+	Printf.fprintf c " > ";
+	print_stp_coq2 c b false;
+	if p then Printf.fprintf c ")"
+      end
+
+let rec print_pretrm_coq2 c m lp rp =
+  match m with
+  | PName x ->
+      Printf.fprintf c "%s" x
+  | PType -> Printf.fprintf c "Type"
+  | PPropTp -> Printf.fprintf c "prop"
+  | PIndTp ->
+      Printf.fprintf c "set"
+  | PTrue -> Printf.fprintf c "True"
+  | PFalse -> Printf.fprintf c "False"
+  | PAp(PNeg,m1) ->
+      if ((lp < 0) && (rp < 30)) then
+	begin
+	  Printf.fprintf c "~ ";
+	  print_pretrm_coq2 c m1 30 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(~ ";
+	  print_pretrm_coq2 c m1 30 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PForall,m1) ->
+      if ((lp < 0) && (rp < 0)) then
+	begin
+	  Printf.fprintf c "(fun p => forall x, p x) ";
+	  print_pretrm_coq2 c m1 (-1) (-1);
+	end
+      else
+	begin
+	  Printf.fprintf c "((fun p => forall x, p x) ";
+	  print_pretrm_coq2 c m1 (-1) (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PExists,m1) ->
+      if ((lp < 0) && (rp < 0)) then
+	begin
+	  Printf.fprintf c "ex _ ";
+	  print_pretrm_coq2 c m1 (-1) (-1);
+	end
+      else
+	begin
+	  Printf.fprintf c "(ex _ ";
+	  print_pretrm_coq2 c m1 (-1) (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(PImplies,m1),m2) ->
+      if ((lp < 17) && (rp < 16)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 17;
+	  Printf.fprintf c " -> ";
+	  print_pretrm_coq2 c m2 16 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 17;
+	  Printf.fprintf c " -> ";
+	  print_pretrm_coq2 c m2 16 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(PRevImplies,m2),m1) ->
+      if ((lp < 17) && (rp < 16)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 17;
+	  Printf.fprintf c " -> ";
+	  print_pretrm_coq2 c m2 16 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 17;
+	  Printf.fprintf c " -> ";
+	  print_pretrm_coq2 c m2 16 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(PAnd,m1),m2) ->
+      if ((lp < 21) && (rp < 20)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 21;
+	  Printf.fprintf c " /\\ ";
+	  print_pretrm_coq2 c m2 20 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 21;
+	  Printf.fprintf c " /\\ ";
+	  print_pretrm_coq2 c m2 20 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(POr,m1),m2) ->
+      if ((lp < 19) && (rp < 18)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 19;
+	  Printf.fprintf c " \\/ ";
+	  print_pretrm_coq2 c m2 18 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 19;
+	  Printf.fprintf c " \\/ ";
+	  print_pretrm_coq2 c m2 18 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(PIff,m1),m2) ->
+      if ((lp < 14) && (rp < 14)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 14;
+	  Printf.fprintf c " <-> ";
+	  print_pretrm_coq2 c m2 14 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 14;
+	  Printf.fprintf c " <-> ";
+	  print_pretrm_coq2 c m2 14 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(PEq,m1),m2) ->
+      if ((lp < 40) && (rp < 40)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 40;
+	  Printf.fprintf c " = ";
+	  print_pretrm_coq2 c m2 40 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 40;
+	  Printf.fprintf c " = ";
+	  print_pretrm_coq2 c m2 40 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAp(PAp(PNEq,m1),m2) ->
+      if (rp >= 40) then Printf.fprintf c "(";
+      Printf.fprintf c "~ (";
+      print_pretrm_coq2 c m1 (-1) 40;
+      Printf.fprintf c " = ";
+      print_pretrm_coq2 c m2 40 (-1);
+      Printf.fprintf c ")";
+      if (rp >= 40) then Printf.fprintf c ")";
+  | PAp(PAp(PNIff,m1),m2) ->
+      if (rp >= 30) then Printf.fprintf c "(";
+      Printf.fprintf c "~ (";
+      print_pretrm_coq2 c m1 (-1) 14;
+      Printf.fprintf c " <-> ";
+      print_pretrm_coq2 c m2 14 (-1);
+      Printf.fprintf c ")";
+      if (rp >= 30) then Printf.fprintf c ")";
+  | PAp(PAp(PNAnd,m1),m2) ->
+      if (rp >= 30) then Printf.fprintf c "(";
+      Printf.fprintf c "~ (";
+      print_pretrm_coq2 c m1 (-1) 20;
+      Printf.fprintf c " //\ ";
+      print_pretrm_coq2 c m2 21 (-1);
+      Printf.fprintf c ")";
+      if (rp >= 30) then Printf.fprintf c ")";
+  | PAp(PAp(PNOr,m1),m2) ->
+      if (rp >= 30) then Printf.fprintf c "(";
+      Printf.fprintf c "~ (";
+      print_pretrm_coq2 c m1 (-1) 18;
+      Printf.fprintf c " \\/ ";
+      print_pretrm_coq2 c m2 19 (-1);
+      Printf.fprintf c ")";
+      if (rp >= 30) then Printf.fprintf c ")";
+  | PIff -> Printf.fprintf c "iff"
+  | POr -> Printf.fprintf c "or"
+  | PAnd -> Printf.fprintf c "and"
+  | PNeg -> Printf.fprintf c "not"
+  | PForall ->
+      Printf.fprintf c "(fun p => forall x, p x)";
+  | PExists ->
+      Printf.fprintf c "(ex _)";
+  | PEq ->
+      Printf.fprintf c "(eq _)";
+  | PAp(m1,m2) ->
+      if ((lp < 5000) && (rp < 5001)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 5000;
+	  Printf.fprintf c " ";
+	  print_pretrm_coq2 c m2 5001 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 5000;
+	  Printf.fprintf c " ";
+	  print_pretrm_coq2 c m2 5001 (-1);
+	  Printf.fprintf c ")";
+	end
+  | PAr(m1,m2) ->
+      if ((lp < 71) && (rp < 70)) then
+	begin
+	  print_pretrm_coq2 c m1 lp 71;
+	  Printf.fprintf c " > ";
+	  print_pretrm_coq2 c m2 70 rp;
+	end
+      else
+	begin
+	  Printf.fprintf c "(";
+	  print_pretrm_coq2 c m1 (-1) 71;
+	  Printf.fprintf c " > ";
+	  print_pretrm_coq2 c m2 70 (-1);
+	  Printf.fprintf c ")";
+	end
+  | POf(n,p) -> print_pretrm_coq2 c n lp rp
+(***  | PDef(n,p) -> print_pretrm_coq2 c n lp rp ***)
+  | PULam(xl,m) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	begin
+	  Printf.fprintf c "fun";
+	  print_ulam_coq2 c xl m
+	end;
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | PLam(xl,m) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	begin
+	  Printf.fprintf c "fun";
+	  print_lam_coq2 c xl m
+	end;
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | PAll(xl,m) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	begin
+	  Printf.fprintf c "forall";
+	  print_all_coq2 c xl m
+	end;
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | PEx(xl,m) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	begin
+	  print_ex_coq2 c xl m
+	end;
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | PChoice((x,xa),m) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	begin
+	  Printf.fprintf c "Eps _ (fun ";
+	  Printf.fprintf c " %s" x;
+	  Printf.fprintf c ":";
+	  print_pretrm_coq2 c xa (-1) (-1);
+	  Printf.fprintf c " => ";
+	  print_pretrm_coq2 c m (-1) (-1);
+	  Printf.fprintf c ")";
+	end;
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | PLet(x,m1,m2) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	Printf.fprintf c "let %s := " (if (x = "") then "_" else x);
+	print_pretrm_coq2 c m1 (-1) (-1);
+	Printf.fprintf c " in ";
+	print_pretrm_coq2 c m2 (-1) (-1);
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | PTLet(x,a1,m1,m2) ->
+      begin
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c "(";
+	Printf.fprintf c "let %s : " (if (x = "") then "_" else x);
+	print_pretrm_coq2 c a1 (-1) (-1);
+	Printf.fprintf c " := ";
+	print_pretrm_coq2 c m1 (-1) (-1);
+	Printf.fprintf c " in ";
+	print_pretrm_coq2 c m2 (-1) (-1);
+	if ((lp >= 0) || (rp >= 0)) then Printf.fprintf c ")";
+      end
+  | _ -> raise (GenericSyntaxError ("Unknown pretrm case print Coq version : " ^ (pretrm_str m)))
+and print_ulam_coq2 c xl m =
+  match xl with
+  | (x::xr) when x = "" ->
+      begin
+	Printf.fprintf c " _";
+	print_ulam_coq2 c xr m;
+      end
+  | (x::xr) ->
+      begin
+	let y = x in
+	Printf.fprintf c " "; Printf.fprintf c "%s" y;
+	print_ulam_coq2 c xr m;
+      end
+  | [] ->
+      begin
+	Printf.fprintf c " => ";
+	print_pretrm_coq2 c m (-1) (-1)
+      end
+and print_lam_coq2 c xl m =
+  match xl with
+  | ((x,a)::xr) when x = "" ->
+      begin
+	Printf.fprintf c " (_"; Printf.fprintf c ":"; print_pretrm_coq2 c a (-1) (-1); Printf.fprintf c ")";
+	print_lam_coq2 c xr m;
+      end
+  | ((x,a)::xr) ->
+      begin
+	let y = x in
+	Printf.fprintf c " ("; Printf.fprintf c "%s" y; Printf.fprintf c ":"; print_pretrm_coq2 c a (-1) (-1); Printf.fprintf c ")";
+	print_lam_coq2 c xr m;
+      end
+  | [] ->
+      begin
+	Printf.fprintf c " => ";
+	print_pretrm_coq2 c m (-1) (-1)
+      end
+and print_all_coq2 c xl m =
+  match xl with
+  | ((x,a)::xr) when x = "" ->
+      begin
+	Printf.fprintf c " (_"; Printf.fprintf c ":"; print_pretrm_coq2 c a (-1) (-1); Printf.fprintf c ")";
+	print_all_coq2 c xr m;
+      end
+  | ((x,a)::xr) ->
+      begin
+	let y = x in
+	Printf.fprintf c " ("; Printf.fprintf c "%s" y; Printf.fprintf c ":"; print_pretrm_coq2 c a (-1) (-1); Printf.fprintf c ")";
+	print_all_coq2 c xr m;
+      end
+  | [] ->
+      begin
+	Printf.fprintf c ", ";
+	print_pretrm_coq2 c m (-1) (-1)
+      end
+and print_ex_coq2 c xl m =
+  match xl with
+  | ((x,a)::xr) when x = "" ->
+      begin
+	Printf.fprintf c "exists _:"; print_pretrm_coq2 c a (-1) (-1); Printf.fprintf c ", ";
+	print_ex_coq2 c xr m;
+      end
+  | ((x,a)::xr) ->
+      begin
+	let y = x in
+	Printf.fprintf c "exists"; Printf.fprintf c " %s" y; Printf.fprintf c ":"; print_pretrm_coq2 c a (-1) (-1); Printf.fprintf c ", ";
+	print_ex_coq2 c xr m;
+      end
+  | [] ->
+      begin
+	print_pretrm_coq2 c m (-1) (-1)
+      end
+
 (*** Shifting, Substitution, Normalization ***)
 
 let rec shift m i j =
@@ -606,6 +1036,13 @@ let rec termNotFreeIn m i =
   | Lam(a,m1) -> termNotFreeIn m1 (i + 1)
   | _ -> true
 
+let rec termNotFreeInL m il =
+  match m with
+    DB(j,_) when (List.mem j il) -> false
+  | Ap(m1,m2) -> (termNotFreeInL m1 il) && (termNotFreeInL m2 il)
+  | Lam(a,m1) -> termNotFreeInL m1 (List.map (fun i -> i + 1) il)
+  | _ -> true
+
 let rec termNoDBs_rec m i =
   match m with
     DB(j,_) -> if (j < i) then true else false
@@ -781,6 +1218,9 @@ let expected_tp m a b =
 
 let rec to_trm (h:(string,(trm * stp) * bool ref) Hashtbl.t) (g:ctx) (m:pretrm) (a:stp option) =
   match m with
+  | PAp(PName "Eps",a) -> let b = to_stp a in (Choice b,Ar(Ar(b,Prop),b))
+  | PName "In" -> let a = Base "set" in (Name("In",Ar(a,Ar(a,Prop))),Ar(a,Ar(a,Prop)))
+  | PName "Subq" -> let a = Base "set" in (Lam(a,Lam(a,Ap(Forall(a),Lam(a,Ap(Ap(Imp,Ap(Ap(Name("In",Ar(a,Ar(a,Prop))),DB(0,a)),DB(2,a))),Ap(Ap(Name("In",Ar(a,Ar(a,Prop))),DB(0,a)),DB(1,a))))))),Ar(a,Ar(a,Prop)))
   | PName x ->
       begin(*** look in g, then in h, then fail ***)
 	try
@@ -895,6 +1335,12 @@ let rec to_trm (h:(string,(trm * stp) * bool ref) Hashtbl.t) (g:ctx) (m:pretrm)
 	    (Ap(n1,n2),a1b)
 	| _ -> raise (GenericSyntaxError("Non-function applied: " ^ (trm_str n1)))
       end
+  | PULam(xl,m1) ->
+      begin
+	match a with
+	| Some(aa) -> to_ulam h g xl m1 aa
+	| None -> raise (GenericSyntaxError("Cannot infer type omitted from lambda"))
+      end
   | PLam(xl,m1) ->
       to_lam h g xl m1 a
   | PAll(xl,m1) ->
@@ -903,6 +1349,13 @@ let rec to_trm (h:(string,(trm * stp) * bool ref) Hashtbl.t) (g:ctx) (m:pretrm)
   | PEx(xl,m1) ->
       expected_tp m a Prop;
       (to_exists h g xl m1,Prop)
+  | PExU(x,a1,m1) ->
+      expected_tp m a Prop;
+      begin
+	let atp = to_stp a1 in
+	match to_trm h ((x,atp)::g) m1 (Some Prop) with
+	| (m1a,_) -> (Ap(Lam(Ar(atp,Prop),Ap(Exists(atp),Lam(atp,Ap(Ap(And,Ap(DB(1,Ar(atp,Prop)),DB(0,atp))),Ap(Forall(atp),Lam(atp,Ap(Ap(Imp,Ap(DB(2,Ar(atp,Prop)),DB(0,atp))),Ap(Ap(Eq(atp),DB(0,atp)),DB(1,atp))))))))),Lam(atp,m1a)),Prop)
+      end
   | PChoice((x,xa),m1) ->
       let xaa = to_stp xa in
       let n1 = to_trm_1 h ((x,xaa)::g) m1 (Some Prop) in
@@ -913,8 +1366,32 @@ let rec to_trm (h:(string,(trm * stp) * bool ref) Hashtbl.t) (g:ctx) (m:pretrm)
       to_trm h g m1 (Some b)
   | PDef(m1,_) ->
       to_trm h g m1 a
+  | PLet(x,m1,m2) ->
+      begin
+	let (m1a,aa) = to_trm h g m1 None in
+	let (m2b,bb) = to_trm h ((x,aa)::g) m2 a in
+	(Ap(Lam(aa,m2b),m1a),bb)
+      end
+  | PTLet(x,a1,m1,m2) ->
+      begin
+	let (m1a,aa) = to_trm h g m1 (Some (to_stp a1)) in
+	let (m2b,bb) = to_trm h ((x,aa)::g) m2 a in
+	(Ap(Lam(aa,m2b),m1a),bb)
+      end
   | _ -> raise (GenericSyntaxError ("Ill-formed term " ^ (pretrm_str m)))
 and to_trm_1 h g m a = let (n,_) = to_trm h g m a in n
+and to_ulam h (g:ctx) xl m a =
+  match xl with
+    (x::xr) ->
+      begin
+	match a with
+	  Ar(a1,a2) ->
+	    let (n1,_) = to_ulam h ((x,a1)::g) xr m a2 in
+	    (Lam(a1,n1),a)
+	| _ ->
+	    raise (ExpectedTypeError(m,a,Ar(Base("_"),Base("*"))))
+      end
+  | [] -> to_trm h g m (Some a)
 and to_lam h (g:ctx) xl m a =
   match xl with
     ((x,xa)::xr) ->
