@@ -14,13 +14,25 @@ type pretrm =
   | PInt of int
   | PReal of float
   | PAp of pretrm * pretrm
+  | PULam of string list * pretrm
   | PLam of ((string * pretrm) list) * pretrm
   | PAll of ((string * pretrm) list) * pretrm
   | PEx of ((string * pretrm) list) * pretrm
+  | PExU of string * pretrm * pretrm
   | PChoice of (string * pretrm) * pretrm
   | PAr of pretrm * pretrm
   | POf of pretrm * pretrm
   | PDef of pretrm * pretrm
+  | PLet of string * pretrm * pretrm
+  | PTLet of string * pretrm * pretrm * pretrm
+
+type prectx = (string option * pretrm option * pretrm option) list
+
+val prectx_lam : prectx -> pretrm -> pretrm
+
+val prectx_all : prectx -> pretrm -> pretrm
+
+val prectx_ar : prectx -> pretrm -> pretrm
 
 val pretrm_str:pretrm -> string
 
@@ -66,6 +78,8 @@ val ap:trm * trm -> trm
 val coqify_name : string -> (string,string) Hashtbl.t -> (string,unit) Hashtbl.t -> string
 val print_stp_coq:out_channel -> stp -> (string,string) Hashtbl.t -> bool -> unit
 val print_pretrm_coq:out_channel -> pretrm -> (string,string) Hashtbl.t -> (string,unit) Hashtbl.t -> int -> int -> unit
+val print_stp_coq2:out_channel -> stp -> bool -> unit
+val print_pretrm_coq2:out_channel -> pretrm -> int -> int -> unit
 
 val shift:trm -> int -> int -> trm
 val subst:trm -> int -> trm -> trm
@@ -73,6 +87,7 @@ val simulsubst:trm -> trm list -> trm
 val namesubst:trm -> string -> trm -> trm
 val gen_lam_body:stp -> trm -> trm
 val termNotFreeIn:trm -> int -> bool
+val termNotFreeInL:trm -> int list -> bool
 val termNoDBs:trm -> bool
 val norm1:(string,trm) Hashtbl.t -> trm -> trm * bool
 val norm:(string,trm) Hashtbl.t -> trm -> trm
